﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ParallelDemo
{
    public static class Utilities
    {
        // Return true if number is prime
        public static bool IsPrime(int number)
        {
            // Check common cases first
            if (number < 2)
                return false;
            else if (number < 4)
                return true;
            else if (number % 2 == 0)
                return false;
            else if (number % 3 == 0)
                return false;

            int divisor = 5;
            int increment = 2;

            // Don't go past the sqrt of the number
            int maxDivisor = (int)Math.Sqrt(number);

            while (divisor < maxDivisor)
            {
                if (number % divisor == 0)
                    return false;

                divisor += increment;
                // Change increment from 2 - 4
                // Since we're past 3, we can just check in these increments:
                //     5, 7, 11, 13, 17, 19, 23, 25, 29, 31, 35, 37, ...
                increment = 6 - increment;
            }
            return true;
        }
        
        // Determine how long it takes to run a method
        public static long Timings(string methodName, Action method)
        {
            Stopwatch sw = new Stopwatch();

            Console.WriteLine("---------------");
            Console.WriteLine("Starting {0}", methodName);

            sw.Start();
            method();
            sw.Stop();

            Console.WriteLine("   Executed in {0} ms", sw.ElapsedMilliseconds);
            Console.WriteLine("---------------");

            return sw.ElapsedMilliseconds;
        }
    }
}
