﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ParallelDemo
{
    class Program
    {
        static void Main(string[] args)
        {
            // Create a large list of numbers
            var numbers = Enumerable.Range(0, 5000000).ToList();

            // Get the total number of primes
            var serial = Utilities.Timings("Serial", () =>
                {
                    int total = 0;
                    foreach (var number in numbers)
                    {
                        if (Utilities.IsPrime(number))
                            ++total;
                    }
                    Console.WriteLine("Number of primes = {0}", total);
                });

            // Get the total number of primes, in a parallel fashion
            var parallel = Utilities.Timings("Parallel", () =>
            {
                int total = 0;
                Parallel.ForEach(numbers, number =>
                {
                    if (Utilities.IsPrime(number))
                        Interlocked.Increment(ref total); // Interlocked works atomically, to avoid locking
                });
                Console.WriteLine("Number of primes = {0}", total);
            });

            Console.WriteLine();
            if (parallel != 0)
                Console.WriteLine("Improvement {0}x", (double)serial / (double)parallel);

            #region Issues: False Sharing Issues
#if false
            // This is very processor/computer specific - on some systems, parallel is a lot slower than serial
            int cores = Environment.ProcessorCount;
            int[] results = new int[cores];
            Utilities.Timings("Serial (False Sharing)", () =>
                {
                    for (int i = 0; i < cores; ++i)
                        for (int j = 0; j < 50000000; ++j)
                            results[i] += 3;

                });
            results = new int[cores];
            Utilities.Timings("Parallel (False Sharing)", () =>
                {
                    Parallel.For(0, cores, i =>
                    {
                        for (int j = 0; j < 50000000; ++j)
                            results[i] += 3;
                    });
                });
#endif

            #endregion


            #region Issues: Inappropriate Closures

#if false
            for (int i = 0; i < 20; ++i)
            {
                // Use temporary to avoid
                int tmp = i;
                Task.Factory.StartNew(() =>
                    {
                        Console.WriteLine("Using temporary {0}", tmp);
                    });
                Task.Factory.StartNew(() =>
                    {
                        Console.WriteLine("Using variable directly {0}", i);
                    });
            }
         
#endif

            #endregion

            Console.WriteLine();
            Console.WriteLine("Press any key to exit....");
            Console.ReadKey();
        }


    }
}
